% New code to solve Nash equilibrium by finding zero graient directly
%   for profit respect to xp, x1 and x2
% Assume duoploy market - 10 producers, each makes only one SI vehicle
% Code by Norman Shiau, 1/26/07

format compact
format short
clear all
close all
clc

global enginebaseSI
global B1 B2 B3 B4 B5 B6 B7 beta
global pscale costInvest costBody mktSize
global K J NX NZ T delta NDRAW ev0

ev0=1;  % ev0 = exp(utility of outside good)
delta=1e-6; % delta for numerical fintie differentiation
pscale=1e4;  % price scale 10,000 (for both wholesale and retail prices)
K=2; % number of producers in the market
J=1; % number of products for each producer
T=1; % number of common retailers in the market
NX=2;% number of variables for each product: [x1, x2] ==> NX+1(w=p,x1,x2) 
NZ=3;% # of product attributes [p, z1, z2]
NDRAW=1000;  % # of random draws (MNL #=1)
solcount=1;

%% Parameters in model
B1 = -2.86e-4; B2 = -0.339; B3 = 0.375; % B&M's logit coefficients
B4 = 670.51; B5 = 0.0063; B6 = 26.23; B7 = 1642.8;
enginebaseSI = 102;  % SI engine base power (kW)
costInvest=0;
%costInvest = 550e6; % Investment cost per vehicle
costBody = 7500;  % Rest of vehicle manufacturing cost
mktSize = (11e6)/7; % Market size (unit)
% coefficients from Boyd and Mellman's Hedonic demand model
% to be noticed that these coefficients are log parameter
% so the beta is actually log-normal distribution

%% code to create random betas  (B&M's MXL)
%mu   =[-7.94,-1.280,-1.75];
%sigma=[ 1.18, 0.001, 1.34];
%randraw=randn(1,NZ,NDRAW); % random normal-distribution draws
%beta=exp(repmat(mu,[1,1,NDRAW])+repmat(sigma,[1,1,NDRAW]).*randraw);
%beta_mean=exp(mu+(sigma.^2)/2); beta=beta_mean;
%beta(:,1:2,:)=-beta(:,1:2,:);
%save randombeta.mat beta
%mean(beta,3)

%% load existing random beta from file
load beta1k.mat
%beta=[B1,B2,B3]; % 
%beta=repmat([B1,B2,B3],[1,1,NDRAW]);
%beta=[-0.0007 -0.2780 0.4235]; % usual mean of 1k draws
%beta(1,:,1)=[-0.0007  -0.2  2.0;];
%beta(1,:,2)=[-0.0007  -2.0  0.2;];

options = optimset('Display','iter','LargeScale','off', ...
  'Algorithm','active-set','TolFun',1e-6,'TolX',1e-6,'TolCon',1e-6);
LB = repmat([0;1.0;0.8;  0;  0;  0;  0],K,1);
UB = repmat([5;3.0;1.3;Inf;Inf;Inf;Inf],K,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SECTION I: Find FOC solutions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% for grid start points
for xw1_0=1.0:.5:2.5,
    for xw2_0=1.0:.5:2.5,
        for x11_0=1:.5:2.5,
            for x12_0=1:.5:2.5,
                for x21_0=0.8:.4:1.2,
                    for x22_0=0.8:.4:1.2,
                        
                        % starting point
                        %x0 = repmat([xw_1;x1_1;x2_1;0;0;0;0],K,1);
                        
                        % Lagrangian multiplier > 0: active constraint
                        x0=[xw1_0;x11_0;x21_0;0;0;0;0;...
                           xw2_0;x12_0;x22_0;0;0;0;0;];
                        disp([xw1_0,x11_0,x21_0,xw2_0,x12_0,x22_0]);
                        
                        % for single start point
%                         x0 = [1.7;2.2;1.2;0;0;0;0;...
%                               1.7;2.2;1.2;0;0;0;0;];
                        
                        % for random start points
                        % x0=LB+(UB-LB).*rand(K*(NX+2),1);
                        % x0(1:NX+2,1)=LB(1:NX+2,1)+(UB(1:NX+2,1)-LB(1:NX+2,1)).*rand(NX+2,1); % NX+2: w, x1, x2, m
                        % x0(NX+3:2*NX+4,1)=UB(1:NX+2,1)-x0(1:NX+2,1); % NX+2: w, x1, x2, m
                        
                        %disp(sprintf('-%d-',tryrun));
                        %disp(x0');
                        %% Call solver to find roots
                        options = optimset('Display','off','Algorithm','active-set','LargeScale','off');
                        [xsol,fval,exitflag,output,lambda,grad,hessian] = fmincon(@obj0,x0,...
                            [],[],[],[],LB,UB,@eqfuns,options);
                        %[xsol,fval,exitflag]=fsolve(@eqfuns,x0);
                        
                        %% Display solutions of producer
                        if exitflag >= 1
                            % If the solution meets necessary conditions
                            xx=reshape(xsol,[(1+3*NX),K])';
                            xw=xx(:,1); x1=xx(:,2); x2=xx(:,3);
                            w=pscale.*xw; p=w; pw=enginebaseSI.*x1;
                            z=getz(xx); z1=z(:,2); z2=z(:,3);
                            [profitM,pr,cv]=estprofit(xx,z);
                            mr=zeros(K,1);
                            profitR=mr.*pr*mktSize;
                            disp('P# WSprice EngScale fdRatio Rmargin Rprice  mpg   acelTime  MktShr   costV Mprofit  Rprofit');
                            for k=1:K
                                disp(sprintf('%2d\t%0.0f\t%1.2f\t%1.2f\t%0.0f\t%0.0f\t%2.1f\t%0.2f\t%0.2f\t%0.0f\t%3.0f\t%3.0f',...
                                    k,w(k),x1(k),x2(k),mr(k),p(k),z1(k),z2(k),pr(k),cv(k),profitM(k)/1e6,profitR(k)/1e6));
                            end
                            solsets(:,:,solcount)=[xx,z1,z2,pr,cv,profitM,profitR];
                            % index: xx1:4,z1:5,z2:6,pr:7
                            solcount=solcount+1;
                        end
                        
                    end
                end
            end
        end
    end
end

size(solsets)
save sols.mat solsets

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SECTION II. Verify Nash conditions for all FOC solutions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NSOL=size(solsets,3);  % get # of FOC solutions

xmfrlb = [0;0.5;0.5]; xmfrub = [5;3.0;1.5]; % bounds for mfgr w & x
%xrtmlb = 0;  xrtmub = 3; % bounds for retialer margins

passncindex=[];
for solindex=1:NSOL
    
    XX0=solsets(:,1:3,solindex);
    profitM=solsets(:,12,solindex);
    profitR=solsets(:,13,solindex);
    
    k=1;
    while(k<=K)
        failcountM=0;
        failcountR=0;
        
        disp(sprintf('Checking NC for sol#%d-k#%d',solindex,k));
        % 1) check NC for manufacturer profit by changing xm, x1, x2
        %    with retailer margin is fixed (for VN only)
        %    using global solver (grid multi-start method)
        % grid multi-start loop
        for xmfr0_w=.5:.5:2.5
            for xmfr0_x1=.8:.8:2.4
                for xmfr0_x2=.7:.3:1.3
                    xmfr0=[xmfr0_w xmfr0_x1 xmfr0_x2];
                    %xmfr0=solsets(k,1:3,solindex);
                    [xmsol,fvalM,exitflag] = fmincon(@objprofitM,xmfr0,...
                        [],[],[],[],xmfrlb,xmfrub,[],options,XX0,k);
                    %disp([round(-fvalM) , round(profitM(k))]);
                    if round(-fvalM) > round(profitM(k)),
                        failcountM=failcountM+1;
                    end
                    
                end
            end
        end
        
        %disp([-fvalM, profitM(k)])
        if failcountM>0,
            disp('MFGR profit NC check failed');
        else
            disp('MFGR profit NC check passed');
        end
        
        
        if failcountM==0,
            k=k+1; % continue to check next guy
            passncindex=[passncindex;solindex];
        else
            k=K+1; % break the while loop to save time
        end
        
    end % end of k while loop
    
end % end of big solindex loop

disp(passncindex');