%% New code to solve Nash equilibrium by finding zero graient directly
%  for profit respect to xp, x1 and x2
% Assume duoploy market - 10 producers, each makes only one SI vehicle
% Code by Norman Shiau, 1/26/07

format compact
clear all
close all
clc

global enginebaseSI
global B1 B2 B3 B4 B5 B6 B7 beta
global pscale costInvest costBody mktSize
global K J NX NZ T delta NDRAW ev0

ev0=exp(0.5);  % ev0 = exp(utility of outside good)
delta=1e-6; % delta for numerical fintie differentiation
pscale=1e4;  % price scale 10,000 (for both wholesale and retail prices)
K=2; % number of producers in the market
J=1; % number of products for each producer
T=1; % number of common retailers in the market
NX=2;% number of variables for each product: [x1, x2] ==> NX+2 for all varibles
NZ=3;% # of product attributes [p, z1, z2]
NDRAW=1000;  % # of random draws (MNL #=1)
solcount=1;

%% Parameters in model
B1 = -2.86e-4; B2 = -0.339; B3 = 0.375; % B&M's logit coefficients
B4 = 670.51; B5 = 0.0063; B6 = 26.23; B7 = 1642.8;
enginebaseSI = 102;  % SI engine base power (kW)
costInvest=0;
%costInvest = 550e6; % Investment cost per vehicle
costBody = 7500;  % Rest of vehicle manufacturing cost
mktSize = (11e6)/7; % Market size (unit)
% coefficients from Boyd and Mellman's Hedonic demand model
% to be noticed that these coefficients are log parameter
% so the beta is actually log-normal distribution

%% code to create random betas
%mu=[-7.94,-1.280,-1.75];
%sigma=[ 1.18, 0.001, 1.34];
%randraw=randn(1,NZ,NDRAW); % random normal-distribution draws
%beta=exp(repmat(mu,[1,1,NDRAW])+repmat(sigma,[1,1,NDRAW]).*randraw);
%beta(:,1:2,:)=-beta(:,1:2,:);
%mean(beta,3)

%% load existing random beta from file
load beta1k.mat
%beta=[-0.0007 -0.2780 0.4235]; % usual mean of 1000 draws
%beta=[B1,B2,B3]; % for base case purpose only
%beta=repmat([B1,B2,B3],[1,1,NDRAW]);
%load twobetas.mat

LB = repmat([ 0;0.5;0.5;0;  0;  0;  0;  0],K,1);
UB = repmat([10;3.0;1.5;50;Inf;Inf;Inf;Inf],K,1);
solsets=[];

%% SECTION 1: find FOC solutions
% multi-start loop to find FOC solutions (comment out to disable)

% uncomment these "for" loops to do multi-start
%for xw1_0=5:.5:5
%    for xw2_0=5:.5:5
%        for xm1_0=16:.5:16
%            for xm2_0=16:.5:.16
%                for x11_0=2:.5:2
%                    for x12_0=2:.5:2
%                        for x21_0=1.1:.1:1.1
%                            for x22_0=1.1:.1:1.1
                                 
    %% starting point
    % x0 for multi-start
    %x0 = [xw1_0;x11_0;x21_0;xm1_0;0;0;100;0;...
    %      xw2_0;x12_0;x22_0;xm2_0;0;0;100;0;];

    % x0 for multi-start (but each firm has same starting point - simpler loop)
    %x0 = repmat([xw_0;x1_0;x2_0;xm_0;0;0;1;0],K,1);

    % a useful x0 for testing the MNL case
    %x0=[1.3;1.7;1.2;.4;0;0;0;0;1.3;1.7;1.2;.4;0;0;0;0;]; 

    % a useful x0 for testing the MXL case
    x0=[5.7;2.7;1.2;23;0;0;0;0;...
        5.7;2.7;1.2;23;0;0;0;0;];

    %x0=reshape(x0',[(NX*3+2)*K,1]);
    %x0 = [xw1_0;x11_0;x21_0;xm2_0;xw2_0;x12_0;x22_0;xm2_0]; % for two guys only
    %x0=LB+(UB-LB).*rand(K*(NX+2),1);
    %x0(1:NX+2,1)=LB(1:NX+2,1)+(UB(1:NX+2,1)-LB(1:NX+2,1)).*rand(NX+2,1); % NX+2: w, x1, x2, m
    %x0(NX+3:2*NX+4,1)=UB(1:NX+2,1)-x0(1:NX+2,1); % NX+2: w, x1, x2, m

    %disp(sprintf('-%d-',tryrun));
    disp(x0');
    %% Call solver to find roots
    options = optimset('Display','iter','LargeScale','off','Algorithm','active-set');
    [xsol,fval,exitflag,output,lambda,grad,hessian] = fmincon(@obj0,x0,...
           [],[],[],[],LB,UB,@eqfuns,options);
    %[xsol,fval,exitflag]=fsolve(@eqfuns,x0);


    %% Display solutions of producer
    if exitflag >= 1
        % If the solution meets necessary conditions
        xx=reshape(xsol,[(3*NX+2),K])';
        xw=xx(:,1); x1=xx(:,2); x2=xx(:,3); xm=xx(:,4);
        w=pscale.*xw; mr=pscale.*xm; p=w+mr; pw=enginebaseSI.*x1; 
        z=getz(xx); z1=z(:,2); z2=z(:,3);
        [profitM,pr,cv]=estprofit(xx,z);
        profitRk=mr.*pr*mktSize;
        disp('P# WSprice EngScale fdRatio Rmargin Rprice  mpg   acelTime  MktShr   costV Mprofit  Rprofit');
        for k=1:K
            disp(sprintf('%2d\t%0.0f\t%1.2f\t%1.2f\t%0.0f\t%0.0f\t%2.1f\t%0.2f\t%0.2f\t%0.0f\t%3.0f\t%3.0f',...
               k,w(k),x1(k),x2(k),mr(k),p(k),z1(k),z2(k),pr(k),cv(k),profitM(k)/1e6,profitRk(k)/1e6));
        end
        solsets(:,:,solcount)=[xw,x1,x2,xm,z1,z2,pr,cv,profitM,profitRk];
        % index: xx1:4,z1:5,z2:6,pr:7
        solcount=solcount+1;
    end % end of if loop


% uncomment these "end" to do multi-start loops
%                            end
%                        end
%                    end
%                end
%            end
%        end
%    end
%end


size(solsets)
save sols.mat solsets
NSOL=size(solsets,3);  % get # of FOC solutions

xmfrlb = [0;1.0;0.8]; xmfrub = [10;3.0;1.3]; % bounds for mfgr w & x
xrtmlb = zeros(K*J,1);  xrtmub = 20*ones(K*J,1); % bounds for retialer margins
options = optimset('Display','off','LargeScale','off','Algorithm','active-set');

passncindex=[];
%% SECTION II. Verify Nash conditions for all FOC solutions
for solindex=1:NSOL
    
    XX0=solsets(:,1:4,solindex);
    profitM=solsets(:,9,solindex);
    profitR=solsets(:,10,solindex);
    profitSCR=sum(profitR,1);
 
    failcountM=0;
    failcountR=0;
    % 1) check NC for retailer profit by changing retailer margin 
    %    with manufacturer's w & x are fixed
    %    using global solver (grid multi-start method)
    %    ** this is for single common retailer (SCR) **
    %    maximum retialer profit check is outside of K loop
    %for xrtm0_xm=1:1:4  % multi-start loop
        xrtm0_xm=23;
        xrtm0=xrtm0_xm*ones(K*J,1);
        [xrtmsol,fvalR,exitflag]=fmincon(@objprofitSCR,xrtm0,...
        [],[],[],[],xrtmlb,xrtmub,[],options,XX0);
        %disp([round(-fvalR),round(profitSCR)]);
        if round(-fvalR) > round(profitSCR),
             failcountR=failcountR+1;
        end
    %end
    
    %disp([-fvalR, profitR(k)])
    if failcountR>0,
        disp(sprintf('Retailer profit NC check for sol#%d failed',solindex));
    else
        disp('Common retailer profit NC check passed');
        % go into MFGR NC check
        k=1;
        while(k<=K)
        
            disp(sprintf('Checking MFGR NC for sol#%d-k#%d',solindex,k));
            % 2) check NC for manufacturer profit by changing xm, x1, x2
            %    with retailer margin is fixed (for VN only)
            %    using global solver (grid multi-start method)
            for xmfr0_w=5.7:.5:5.7
                for xmfr0_x1=2.8:.5:2.8
                    for xmfr0_x2=1.2:.3:1.2
                        xmfr0=[xmfr0_w xmfr0_x1 xmfr0_x2];
                        %xmfr0=solsets(k,1:3,solindex);
                        [xmsol,fvalM,exitflag] = fmincon(@objprofitM,xmfr0,...
                            [],[],[],[],xmfrlb,xmfrub,[],options,XX0,k);
%                        disp([round(-fvalM) round(profitM(k))]);
                        if round(-fvalM) > round(profitM(k)),
                            failcountM=failcountM+1;
                        end
                    
                    end
                end
            end % end of multi-start loop

            %disp([-fvalM, profitM(k)])
            if failcountM>0,
                disp('MFGR profit NC check failed');
                k=K+1; % set the k>K to break the while loop
            else
                disp('MFGR profit NC check passed');
                k=k+1; % continue to check next manufacturer
            end
            
        end % end of k while loop

    end % end of SCR's if loop

    if failcountM==0 && failcountR==0
        % if all passed, store the set of solution
        passncindex=[passncindex;solindex];
    end
        
end % end of big solindex loop

disp(passncindex');