%% New code to solve Nash equilibrium by using sequential optmization
%  for profit respect to p, x1 and x2
% Assume duoploy market - 10 producers, each makes only one SI vehicle
% Use cubic spline function (ppform) instead of single surface fitting fucntion
% Code by Norman Shiau, 1/19/07
% 7/18/07 N.S. - modified for MXL capability

clear all
clc
%% Global data
global B1 B2 B3 B4 B5 B6 B7
global enginebaseSI pscale
global costInvest costBody mktSize
global K J NX NZ T NDRAW ev0 beta
%global xmfrlb xmfrub

ev0=exp(5);  % ev0 = exp(utility of outside good)
K=2; % number of producers in the market
J=1;   % number of products for the producer
T=1; % number of common retailers in the market
NX=2;% number of variables for each product: [x1,x2] ==> NX+2 for all varibles
NZ=3;% # of product attributes [p, z1, z2]
NDRAW=1000;  % # of random draws (MNL #=1)
pscale=10000;   % price scale factor 10,000

%% Parameters in model
B1 = -2.86e-4; B2 = -0.339; B3 = 0.375;  % B&M's MNL beta parameters
B4 = 670.51; B5 = 0.0063; B6 = 26.23; B7 = 1642.8;
enginebaseSI = 102;  % SI102 engine (kW)
%costInvest = 550e6; % Investment cost per production line
costInvest =0; % assume zero here
costBody = 7500;  % Rest of vehicle manufacturing cost
mktSize = (11e6)/7; % Market size (unit)

%% set beta
%beta=[B1,B2,B3]; % for base case purpose only
%beta=repmat([B1,B2,B3],[1,1,NDRAW]);
load beta1k.mat
%beta=[-0.0007 -0.2780 0.4235]; % usual mean of 1000 draws
%beta(:,:,1)=[-0.0007 -0.278 0.012647];
%beta(:,:,2)=[-0.0007 -0.278 9.0];

%% start of CONTROL loop
nruns=300;

%% define the main data M matrix (3-dimensional)
% size of M: [K,J,15]
%The 3rd element in M matrix is all vehicle data
% 1    = type of engine: 1=SI95
% 2:5  = xw, x1, x2 and xm
% 6:8  = price, mpg, acceltime
% 9    = vehicle cost
% 10   = Market share of this product
% 11   = MFGR profit (profitM)
% 12   = Retailer profit (profitR)
% 13   = exitflagM
% 14   = exitflagR
M=zeros(K,J,14);
M(:,1,1) = 1; % assmue all SI engine

%% initialize M matrix
% 2:5 = xw, x1, x2, xm
LB = repmat([0,1.0,0.8, 0],[K,1]);
UB = repmat([6,3.0,1.3,30],[K,1]);
xx0=LB+(UB-LB).*rand(K,(NX+2)); % random start

%x0 = [1.4, 1, 1, .5]; % given the starting design point here
enginetype=1;

% Establish single product data structure
M(:,1,2:5)=xx0;
xx=xx0;
%xx=reshape(xsol,[(NX+2),K])';
xw=xx(:,1); x1=xx(:,2); x2=xx(:,3); xm=xx(:,4);
w=pscale.*xw; mr=pscale.*xm; p=w+mr; pw=enginebaseSI.*x1;
z=getz(xx); M(:,1,6:8)=z;
% z1=z(:,2); z2=z(:,3);
[profitM,pr,cv]=estprofit(xx,z);
profitR=mr.*pr*mktSize;
M(:,1,9)=cv; M(:,1,10)=pr;
M(:,1,11)=profitM;  M(:,1,12)=profitR;
%%%%% eng of building up vehicle data structure (info at product level)
oldM=M; % oldM is a temporary matrix for convegence check purpose

xmfrlb = [ 0;0.5;0.5];
xmfrub = [10;3.0;1.5]; % bounds for mfgr w & x

% For common retailer, variables are retail margins of all products
xrtmlb = zeros(K*J,1);
xrtmub = 30*ones(K*J,1); % bounds for retialer margins

%% run sequential optimzation to solve the dynamic equilibrium problem
options = optimset('Display','off','LargeScale','off','Algorithm','active-set');
disp('R#-k#-j#   WSprice EngScale fdRatio Rmargin Rprice  mpg  accTime MktShr cost Mprofit Rprofit EFM EFR');

for rr=1:nruns
    %xrtm0=xrtmlb+(xrtmub-xrtmlb).*rand(K*J,1); % random start
    %xrtm0=.3*ones(K*J,1); % initial point for SCR retail margin
    xrtm0=11*ones(K*J,1);
    % 1) Maximize retailer profit by changing retailer margin
    %    here retailer type is **signle common retailer**
    %    with w & x fixed (VN game rule)
    %    SCR's retailer margin decision is outside of K-J loop
    [xrtmsol,nfvalR,exitflagR]=fmincon(@objprofitSCR,xrtm0,...
        [],[],[],[],xrtmlb,xrtmub,[],options,M);
    M(:,1,5)=xrtmsol;
    M(:,1,14)=exitflagR;
    
    for kk=1:K
        for jj=1:J  % [kk jj] is producer & product index
            
            % using fixed single start point
            %xmfr0=[3.5,2.5,1];
            % using random start point
            xmfr0=xmfrlb+(xmfrub-xmfrlb).*rand(3,1); 
            % 2) Maximize manufacturer profit by changign w & x
            %    with retailer margin xm fixed (VN game rule)
            [xmfrsol,nfvalM,exitflagM,output,lambda,grad,hessian] = ...
                fmincon(@objprofitM,xmfr0,[],[],[],[],xmfrlb,xmfrub,...
                [],options,[kk jj],M);
            M(kk,jj,2:4)=xmfrsol;
            M(kk,jj,13)=exitflagM;
            
            % display on screen
            xx=squeeze(M(:,1,2:5));
            xw=xx(:,1); x1=xx(:,2); x2=xx(:,3); xm=xx(:,4);
            w=pscale.*xw; mr=pscale.*xm; p=w+mr; pw=enginebaseSI.*x1;
            z=getz(xx); M(:,1,6:8)=z;
            z1=z(:,2); z2=z(:,3);
            [profitM,pr,cv]=estprofit(xx,z);
            profitR=mr.*pr*mktSize;
            M(:,1,9)=cv; M(:,1,10)=pr;
            M(:,1,11)=profitM; M(:,1,12)=profitR;
            
            disp(sprintf('%2d-%2d-%2d  %0.0f  %1.2f  %1.2f  %0.0f  %0.0f  %2.1f  %2.2f  %0.3f  %0.0f  %0.0f  %0.0f  %1d  %1d',...
                rr,kk,jj,w(kk),x1(kk),x2(kk),mr(kk),p(kk),z1(kk),z2(kk),pr(kk),cv(kk),profitM(kk)/1e6,profitR(kk)/1e6,exitflagM,exitflagR));
        end % end of jj loop
    end % end of kk loop
    
    disp(sprintf('SCR profit = %0.0f',sum(profitR/1e6)));
    % Check whether convergence reached after a complete round
    % by comparing the data with previous one
    xtol=squeeze(abs(oldM(:,:,2:5)-M(:,:,2:5)));
    disp(xtol(kk,:)<1e-6); % display check flag (1: pass) on screen
    tolcheck=sum(sum(xtol<1e-6,2),1);
    
    if tolcheck == K*J*(NX+2),
        disp(sprintf('Nash equilibrium reached at the round #%d',rr));
        break
    end
    
    oldM=M;
end % end of rr loop
%save([savename, '.mat']);
%diary off
