% folder name: VN4MCR
%
% Finially solved the Manufacturer Stackelberg problem!!
% Date: 11/11/2007 midnight, Norman Shiau

format compact
clear all
close all
clc

global enginebaseSI
global B1 B2 B3 B4 B5 B6 B7 beta
global pscale costInvest costBody MktSize
global K NX T R ev0

ev0=1;  % ev0 = exp(utility of outside good)
pscale=1e4;  % price scale 10,000 (for both wholesale and retail prices)
K=2; % number of producers in the market
T=2; % number of common retailers in the market
R=1000;  % # of random draws (MNL #=1)
NX=2;% number of variables for each product: [x1, x2] ==> NX+2 for all varibles
NZ=2;% # of product attributes [z1, z2]
solcount=1; % count for FOC solution number

% Parameters in model
B1 = -2.86e-4; B2 = -0.339; B3 = 0.375; % B&M's MNL beta
B4 = 670.51; B5 = 0.0063; B6 = 26.23; B7 = 1642.8;
enginebaseSI = 102;  % SI engine base power (kW)
costInvest=0;
%costInvest = 550e6; % Investment cost per vehicle
costBody = 7500;  % Rest of vehicle manufacturing cost
MktSize = (11e6)/7; % Market size (unit)

% coefficients from Boyd and Mellman's Hedonic demand model
% to be noticed that these coefficients are log parameter
% so the beta is actually log-normal distribution

% code to create random betas (updated to beta [R,NZ])
%b=[-7.94,-1.280,-1.75];
%w=[ 1.18, 0.001, 1.34];
%beta=exp( repmat(b,[R,1])+repmat(w,[R,1]).*randn(R,1+NZ) );
%beta(:,1:2)=-beta(:,1:2);
%mean(beta,1)

% load existing random beta from file
load beta1000.mat % beta for mixed logit (1000 draws)
%beta=[-0.0007 -0.2780 0.4235]; % usual mean of 1000 draws
%beta=[B1,B2,B3]; % the beta for aggregate logit model

% uncommnet these for-loops to run full-scale multi-start
% for xw1t1_0=1.5:.75:3
%    for xw1t2_0=1.5:.75:3
%       for x11_0=1.5:1:2.5
%          for x21_0=1.2:.5:1.2
%             for xm1t1_0=.5:.5:1
%                for xm1t2_0=.5:.5:1
%                   for xw2t1_0=1.5:.75:3
%                      for xw2t2_0=1.5:.75:3
%                         for x12_0=1.5:1:2.5
%                            for x22_0=1.2:.5:1.2
%                               for xm2t1_0=.5:.5:1
                                  for xm2t2_0=1:.5:1

% SECTION 1: find FOC solutions
xw1t1_0=1.5; xw1t2_0=1.5; x11_0=1.5; x21_0=1.2;   xm1t1_0=.5; xm1t2_0=.5;
xw2t1_0=1.5; xw2t2_0=1.5; x12_0=1.5; x22_0=1.2;   xm2t1_0=.5;

x0=[xw1t1_0;xw1t2_0;x11_0;x21_0;xm1t1_0;xm1t2_0;...
    xw2t1_0;xw2t2_0;x12_0;x22_0;xm2t1_0;xm2t2_0;];

% x0=[3;1.5;2;1.2;2;.8;...
%     3;1.5;2;1.2;.8;2;];
xx0=reshape(x0,[6,K])';
disp(xx0);

% Call solver to find roots
LB = repmat([ 0; 0;1.0;0.8;0;0;],K,1);
UB = repmat([10;10;3.0;1.3;5;5;],K,1);
options = optimset('Display','iter','LargeScale','off', ...
      'Algorithm','active-set','TolCon',1e-10,'TolX',1e-10);
[xsol,fval,exitflag]=fmincon(@objzero,x0,[],[],[],[],LB,UB,@focfuns,options);

% Display solutions of producer
if exitflag >= 1
    xxsol=reshape(xsol,[6,K])';
    xw_r1=xxsol(:,1); xw_r2=xxsol(:,2);
    x1=xxsol(:,3); x2=xxsol(:,4);
    xm_r1=xxsol(:,5); xm_r2=xxsol(:,6);
    xx(1,1,:)=[xw_r1(1),x1(1),x2(1),xm_r1(1)];
    xx(1,2,:)=[xw_r2(1),x1(1),x2(1),xm_r2(1)];
    xx(2,1,:)=[xw_r1(2),x1(2),x2(2),xm_r1(2)];
    xx(2,2,:)=[xw_r2(2),x1(2),x2(2),xm_r2(2)];
    w=pscale.*xx(:,:,1); m=pscale.*xx(:,:,4); p=w+m;
    z=getz(xx); z1=z(:,1); z2=z(:,2);
    % calculate s
    v=getv(p,z); ev=exp(v); % size [K,1]
    evsum=sum(sum(ev,3),2)+ev0; % ev0 = exp(utility of outside good)
    for kk=1:K,
        for tt=1:T
            s(:,kk,tt)=ev(:,kk,tt)./evsum;
        end
    end  % size s : [R,K,T]
    mktshr=squeeze(sum(s,1)/R); % size [K,T]
    cv=getcv(xx);
    profitM=MktSize.*mktshr.*(w-cv);
    profitR=MktSize.*mktshr.*m;

    disp('k-t    w    x1   x2     m     p     z1    z2     s     cv   profitM profitR');
    disp('---  ----- ---- ---- ------ ------ ----- ----- ----- ------ ------- -------');
    for k=1:K
        for t=1:T
            disp(sprintf('%1d-%1d %6.0f %3.2f %3.2f %6.0f %6.0f %4.2f  %4.2f %5.3f %6.0f %6.0fM %6.0fM',...
               k,t,w(k,t),x1(k),x2(k),m(k,t),p(k,t),z1(k),z2(k),mktshr(k,t),cv(k,t),profitM(k)/1e6,profitR(t)/1e6));
        end
    end
    profitMFGR=sum(profitM,2);
    profitMCR=sum(profitR,1);
    for k=1:K
        disp(sprintf('Manufacturer k%1d profit = %6.0f mil$',k,profitMFGR(k)/1e6));
    end
    for t=1:T
        disp(sprintf('    Retailer t%1d profit = %6.0f mil$',t,profitMCR(t)/1e6));
    end
    focsols(solcount).xx=xx;
    focsols(solcount).profitMFGR=profitMFGR;
    focsols(solcount).profitMCR=profitMCR;
    solcount=solcount+1;
end


                                  end

% uncommnet these loop-ends to run multi-start
%                               end
%                            end
%                         end
%                      end
%                   end
%                end
%             end
%          end
%       end
%    end
%end



save solfile.mat focsols
NSOL=size(focsols,2);  % get # of FOC solutions
disp(sprintf('Number of FOC solution found = %d \n',NSOL));

% call checknc to verify Nash condition
checkNC