%% New code to solve Nash equilibrium by using sequential iterative optmization
% Code by Norman Shiau, 1/19/07
% 7/18/07 N.S. - modified for MXL capability

format compact
clear all
close all
clc

%% Global data
global B1 B2 B3 B4 B5 B6 B7
global enginebaseSI pscale
global costInvest costBody mktSize
global K J NX NZ T NDRAW ev0 beta
%global xmfrlb xmfrub

ev0=1;  % ev0 = exp(utility of outside good)
K=2; % number of producers in the market
J=1;   % number of products for the producer
T=2; % number of common retailers in the market
NX=2;% number of variables for each product: [x1,x2] ==> NX+2 for all varibles
NZ=3;% # of product attributes [p, z1, z2]
NDRAW=1;  % # of random draws (MNL #=1)
pscale=10000;   % price scale factor 10,000

%% Parameters in model
B1 = -2.86e-4; B2 = -0.339; B3 = 0.375; % B&M's MNL coefficients
B4 = 670.51; B5 = 0.0063; B6 = 26.23; B7 = 1642.8;
enginebaseSI = 102;  % SI102 engine (kW)
%enginebaseCI = 90.44; % CI88 engine (kW)
%costInvest = 550e6; % Investment cost per production line
costInvest =0; % assume zero here
costBody = 7500;  % Rest of vehicle manufacturing cost
mktSize = (11e6)/7; % Market size (unit)

%% code to create random betas  (B&M's MXL)
%mu   =[-7.94,-1.280,-1.75];
%sigma=[ 1.18, 0.001, 1.34];
%randraw=randn(1,NZ,NDRAW); % random normal-distribution draws
%beta=exp(repmat(mu,[1,1,NDRAW])+repmat(sigma,[1,1,NDRAW]).*randraw);
%beta_mean=exp(mu+(sigma.^2)/2); beta=beta_mean;
%beta(:,1:2,:)=-beta(:,1:2,:);
%mean(beta,3)

%% set beta
beta=[B1,B2,B3]; % for base case purpose only
%beta=repmat([B1,B2,B3],[1,1,NDRAW]);
%load beta1k.mat
%beta=[-0.0007 -0.2780 0.4235]; % usual mean of 1000 draws

%% start of CONTROL loop
nruns=200;

%% define the main data M matrix (3-dimensional)
% MCR: each has same product design but different wholesale and retail prices
% size of M: [K,J,T,15]
%The 4th element in M matrix is all vehicle data
% 1    = type of engine: 1=SI95 (assume all gasoline engine)
% 2:5  = xw, x1, x2 and xm
% 6:8  = price, mpg, acceltime
% 9    = vehicle cost
% 10   = Market share of this product
% 11   = MFGR profit (profitM) for this product
% 12   = Retailer profit (profitR) for this product
% 13   = exitflagM
% 14   = exitflagR
M=zeros(K,J,T,14);
enginetype=1; M(:,1,:,1) = enginetype; % assmue all SI engine

%% initialize M matrix
% 2:5 = xw, x1, x2, xm
lb(1,1,1,1:4)=[0,1.0,0.8,0];
ub(1,1,1,1:4)=[6,3.0,1.5,6]; % xw x1 x2 xm
LB = repmat(lb,[K,1,T,1]);
UB = repmat(ub,[K,1,T,1]);
xx0=LB+(UB-LB).*rand(K,1,T,(NX+2)); % size [K,J,T,NX+2] (2 for xp&xm)

% Establish single product data structure
M(:,1,:,2:5)=xx0;
xx=xx0;
%xx=reshape(xsol,[(NX+2),K])';
xw=xx(:,:,:,1); % size [K,J,T,1]
x1=xx(:,:,:,2);
x2=xx(:,:,:,3);
xm=xx(:,:,:,4);
w=pscale.*xw; mr=pscale.*xm; p=w+mr; pw=enginebaseSI.*x1;

z=getz(xx); M(:,:,:,6:8)=z;
[profitM,pr,cv,profitMP]=estprofitM(xx,z);
% mr & pr size: [K,J,T]
profitR=mr.*pr*mktSize; % retail profit for each product
M(:,:,:,9)=cv;  M(:,:,:,10)=pr;
M(:,:,:,11)=profitMP;  M(:,:,:,12)=profitR;
%%%%% eng of building up vehicle data structure (info at product level)
oldM=M; % oldM is a temporary matrix for convegence check purpose

% *** update 10/4/2007 - Norman Shiau
%   I found the iteration for identical design rule does not converge
%   therefore I decide to change the rule and allow manufacturers
%   change their design for different common retailer

% w x1 x2 : size [1+NX,1] for all common retailer t
xmfrlb = [0;1.0;0.8]; % lower bound
xmfrub = [6;3.0;1.3]; % upper bound

% For each common retailer, variables are retail margins includes of its products
xrtmlb =  zeros(K*J,1); xrtmub =2*ones(K*J,1);
%xrtmlb =  0; xrtmub =3; % trick to get converge quicker

%% run sequential optimzation to solve the dynamic equilibrium problem
options = optimset('Display','off','LargeScale','off', ...
    'Algorithm','active-set', 'TolX',1e-10,'TolFun',1e-10);
nfvalR=0;
exitflagR=0;

for rr=1:nruns
    disp('R#-k#-j#-#t WSprice EngScale fdRatio Rmargin Rprice mpg  accTime MktShr cost Mprofit Rprofit EFM EFR');
    
    for kk=1:K
        for jj=1:J  % [kk jj] is producer & product index
            % update: 10/5/2007 - new idea
            % MFGRs use same wholesale price & design to different commmon
            % retailers
            
            % using a fixed signle start point
            %xmfr0=[1.3;1.5;1.1];% w x1 x2 size [NX+1,1]
            
            % using random start point
            xmfr0=xmfrlb+(xmfrub-xmfrlb).*rand(NX+1,1); 
            % 2) Maximize manufacturer profit by changign w & x
            %    with retailer margin xm fixed (VN game rule)
            %    manufacturer needs to decided x and w_t (for all
            %    common retailers _ t)
            [xmfrsol,nfvalM,exitflagM,output,lambda,grad,hessian] = ...
                fmincon(@objprofitM,xmfr0,[],[],[],[],xmfrlb,xmfrub,...
                [],options,[kk jj],M);
            % copy the solution into M
            M(kk,jj,:,2:4)=repmat(reshape(xmfrsol,[1,1,1,NX+1]),[1,1,T,1]);
            M(kk,jj,:,13)=exitflagM; % given exitflagM
            
            % update M using new xx in M and display on screen
            xx=M(:,:,:,2:5); % size:[K,J,T,4] - xw, x1, x2 and xm
            xw=xx(:,:,:,1); % size [K,J,T,1]
            x1=xx(:,:,:,2);
            x2=xx(:,:,:,3);
            xm=xx(:,:,:,4);
            w=pscale.*xw; mr=pscale.*xm; p=w+mr; pw=enginebaseSI.*x1;
            z=getz(xx); M(:,:,:,6:8)=z;
            [profitM,pr,cv,profitMP]=estprofitM(xx,z); % size z: [K,J,T,NZ]
            profitR=mr.*pr*mktSize; % retail profit for each product
            M(:,:,:,9)=cv;  M(:,:,:,10)=pr;
            M(:,:,:,11)=profitMP;  M(:,:,:,12)=profitR;
            
            z1=z(:,:,:,2); z2=z(:,:,:,3);
            
        end % end of jj loop
    end % end of kk loop
    
    % Loop: competition of common retailers
    for tt=1:T  % for MCR
        %xrtm0=1;
        %xrtm0=xrtmlb+(xrtmub-xrtmlb).*rand(K*J,1); % random start
        xrtm0=[.4;.4];
        % Maximize retailer profit by changing retailer margin
        %   here retailer is ** one of multiple common retailers **
        %   with w & x fixed (VN game rule)
        %   SCR's retailer margin decision is outside of K-J loop
        [xrtmsol,nfvalR,exitflagR]=fmincon(@objprofitMCR,xrtm0,...
            [],[],[],[],xrtmlb,xrtmub,[],options,tt,M);
        
        M(:,:,tt,5)=reshape(xrtmsol,[K,J]);
        %        M(:,:,tt,5)=xrtmsol; % trick mode
        %        M(:,:,tt,5)=xrtmsol;
        M(:,:,tt,12)=-nfvalR;
        M(:,:,tt,14)=exitflagR;
    end % end of loop t
    
    
    for kk=1:K
        for jj=1:J
            for tt=1:T
                disp(sprintf('%2d-%2d-%2d-%2d\t%0.0f\t%1.2f\t%1.2f\t%0.0f\t%0.0f\t%2.1f\t%2.1f\t%0.3f\t%0.0f\t%0.0f\t%0.0f\t%1d\t%1d',...
                    rr,kk,jj,tt,w(kk,jj,tt),x1(kk,jj,tt),x2(kk,jj,tt),...
                    mr(kk,jj,tt),p(kk,jj,tt),z1(kk,jj,tt),z2(kk,jj,tt),...
                    pr(kk,jj,tt),cv(kk,jj,tt),profitMP(kk,jj,tt)/1e6,...
                    profitR(kk,jj,tt)/1e6,exitflagM,exitflagR));
            end
        end
    end
    disp(sprintf('Profit of MFGR k1=%0.0f k2=%0.0f',profitM(1)/1e6,profitM(2)/1e6));
    disp(sprintf('Profit of MCR t1=%0.0f t2=%0.0f',sum(profitR(:,1,1))/1e6,sum(profitR(:,1,2))/1e6));
    
    
    % Check whether convergence reached after a complete round
    % by comparing the data with previous one
    xtol=squeeze(abs(oldM(:,:,:,2:5)-M(:,:,:,2:5)));
    % old: disp(xtol(kk,:)<1e-6);
    % display check flag (1: pass) on screen
    tolcheck=sum(sum(sum(xtol<1e-7,3),2),1);
    disp(tolcheck);
    
    if tolcheck == K*J*T*(NX+2),
        disp(sprintf('Nash equilibrium reached at the round #%d',rr));
        break
    end
    
    oldM=M;
end % end of rr loop
%save([savename, '.mat']);
%diary off
